const lo = require('lodash');
const hlp = require('../../../helpers');
const hlpPolicy = require('../../../helpers_policy');
const hlpSch = require('./TaskScheduleHelper.js');
const utils    = require('../../../utils');
const utilsPep = require('../../utils/pep.js');

const defPolicySections = ['ApplicationControlPolicy','PredefinedTasksPolicy'];
const formCommonValues = ['EnforsmentMode','UseCache','BlockInterperetesWhenCliEmpty',
    'UseBinaryRules','MonitorDllLoading','UseScriptOrMsiRules',
    'BlockUntrustedApplication', 'AllowTrustedApplication','AllowUserOrGroups'];
const formRulesValues = ['RuleUsageStrategy','RuleCollection','AdditionalRuleCollection'];
const formSdcValues = ['UseTrustedInstaller', 'TrustWindowsInstaller', 'TrustBackgroundIntelligentTransferService', 'TrustedInstallers'];


function toView(TASK_ADDITIONAL_PARAMS, activePolicyCache, formType) {
    if(formType === 'Schedule' ) {
        return hlpSch.toView(TASK_ADDITIONAL_PARAMS.Schedule, activePolicyCache.PredefinedTasksPolicy.ApplicationControlStartOptions);
    }
    if(formType!=='Main')
        return {}

    let data = {
        Form_Control_PepCommon: hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.ApplicationControlPolicy,formCommonValues),
        Form_Control_PepRules: hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.ApplicationControlPolicy,formRulesValues),
        Form_Control_PepSdc: hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.ApplicationControlPolicy,formSdcValues)
    }

    data.Form_Control_PepCommon.Users = utils.toViewArray(data.Form_Control_PepCommon.AllowUserOrGroups.KLPRSS_Val[0], 'User');
    // rules is mandatory
    const AppCtrlPolicy = activePolicyCache.ApplicationControlPolicy;
    const AppCtrlSetts = TASK_ADDITIONAL_PARAMS.Settings;
    if(AppCtrlPolicy.RuleCollection.KLPRSS_Mnd || AppCtrlPolicy.AdditionalRuleCollection.KLPRSS_Mnd) {
        const binData = AppCtrlPolicy.RuleCollection.KLPRSS_Mnd ? AppCtrlPolicy.RuleCollection.KLPRSS_Val[0].value : AppCtrlSetts.RuleCollection[0].value;
        data.Form_Control_PepRules.uiFields = utilsPep.toViewSettingsRules(binData);
        if(AppCtrlPolicy.AdditionalRuleCollection.KLPRSS_Mnd) {
            const addRules = utilsPep.toViewSettingsRules(AppCtrlPolicy.AdditionalRuleCollection.KLPRSS_Val[0].value);
            data.Form_Control_PepRules.uiFields.Rules = data.Form_Control_PepRules.uiFields.Rules.concat(addRules.Rules);
        }
    }
    else { // collect original rule's list
        data.Form_Control_PepRules.uiFields = utilsPep.toViewSettingsRules(AppCtrlSetts.RuleCollection[0].value);
    }

    data.Form_Control_PepCommon.LocalTask = true;
    data.Form_Control_PepRules.LocalTask = true;
    data.Form_Control_PepSdc.LocalTask = true;
    return data;
}

function toRaw(TASK_ADDITIONAL_PARAMS, changedData, formType) {
    if(formType === 'Schedule' ) {
        hlpSch.toRaw(changedData, TASK_ADDITIONAL_PARAMS.Schedule);
        return;
    }
    if(formType!=='Main')
        return;
    // common
    const Common = hlp.ItemToLocalSettings(changedData.Form_Control_PepCommon);
    Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(Common, formCommonValues));
    // rules
    const bLocked = (changedData.Form_Control_PepRules.KLPRSS_Mnd || changedData.Form_Control_PepRules.AdditionalRuleCollection.KLPRSS_Mnd) ? true : false;
    if(!bLocked)   {
        const ruleCollection = utilsPep.toRowSettingsRules(changedData.Form_Control_PepRules);
        TASK_ADDITIONAL_PARAMS.Settings.RuleCollection = ruleCollection;
        TASK_ADDITIONAL_PARAMS.Settings.AdditionalRuleCollection = utilsPep.GetDummyBinRules();
    }

    // sdc
    const Sdc = hlp.ItemToLocalSettings(changedData.Form_Control_PepSdc);
    Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(Sdc, formSdcValues));
}

function RbacToPolicyCache(activePolicyCache){
    const allFields = formCommonValues.concat(formRulesValues).concat(formSdcValues);
    activePolicyCache.ApplicationControlPolicy = hlpPolicy.SetPolicyFieldsRbacLocked(activePolicyCache.ApplicationControlPolicy, allFields);
    activePolicyCache.PredefinedTasksPolicy = hlpPolicy.SetPolicyFieldsRbacLocked(activePolicyCache.PredefinedTasksPolicy, ['ApplicationControlStartOptions']);
    return activePolicyCache;
}

module.exports = {
    defPolicySections,
    toView,
    toRaw,
    RbacToPolicyCache
};
