const lo = require('lodash');
const hlp     = require('../../../helpers');
const hlpPolicy = require('../../../helpers_policy');
const utils   = require('../../../utils');
const utilsTs = require('../../utils/ts.js');
const hlpSch = require('./TaskScheduleHelper.js');

const defPolicySections = ['WebTrafficMonitorPolicy','PredefinedTasksPolicy'];
const formModeValues = ['ProxyMode','ServerPort','Uri','EnableHttps','TlsVersion','BlockUntrustedCert','ProxyPort','UseExtProxyServer','ExtProxyServerAddress','ExtProxyServerPort'];
const formProtectValues = ['UseHeuristicAnalyzer','HeuristicAnalyzerLevel','ScanArchived','ScanPlainMail','InfectedAction','SuspiciousAction','UseSizeLimit','SizeLimit','UseAdvancedActions'];
const formWebValues = ['UseOutlookPlugin','EnableWmuf','EnableAntiPhishing','UseKsn','UseSandbox'];
const formWcValues = ['UseUrlRules', 'UseCertificates', 'UrlRules', 'CertificateSearchPatterns'];
const formCategorizerValues = ['UseContentCategories', 'ContentCategories'];
const formBaseValues = ['ExcludeThreats','ScanExtensionMasks','ScanOle','ScanPacked','ScanSfxArchived','ScanType','UseTimeLimit','TimeLimit','UseExcludeThreats','UseTrustedZoneSettings'];
const formControlledPortsValues = ['ControlledPorts', 'UsePortMode'];
const formExcAppsValues = ['UseExcludeProcesses','ExcludeProcesses'];

function toView(TASK_ADDITIONAL_PARAMS, activePolicyCache, formType) {
    if(formType === 'Schedule' ) {
        return hlpSch.toView(TASK_ADDITIONAL_PARAMS.Schedule, activePolicyCache.PredefinedTasksPolicy.TrafficMonitorStartOptions);
    }
    else if(formType==='Protection_TS_ControlledPorts'){
        let cPorts = hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.WebTrafficMonitorPolicy,formControlledPortsValues);
        cPorts.uiFields = {Ports: utils.toViewArray(cPorts.ControlledPorts.KLPRSS_Val[0], 'Port')};
        return cPorts;
    }
    else if(formType==='Protection_TS_ExcludedPorts'){
        let ePorts = hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.WebTrafficMonitorPolicy,['ExcludedPorts']);
        ePorts.uiFields = {Ports:utils.toViewArray(ePorts.ExcludedPorts.KLPRSS_Val[0], 'Port')};
        return ePorts;
    }
    else if(formType==='Protection_TS_ExcludedIps'){
        let eIps = hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.WebTrafficMonitorPolicy,['ExcludedAddresses']);
        eIps.uiFields = {Ips: utils.toViewArray(eIps.ExcludedAddresses.KLPRSS_Val[0], 'Ip')};
        return eIps;
    }
    else if(formType==='Protection_TS_ExcludedProcs'){
        let eApps = hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.WebTrafficMonitorPolicy,formExcAppsValues);
        eApps.uiFields = {Apps: utils.toViewArray(eApps.ExcludeProcesses.KLPRSS_Val[0], 'App')};
        return eApps;
    }

    if(formType!=='Main')
        return {};

    const Settings = lo.clone(TASK_ADDITIONAL_PARAMS.Settings);

    let data = {
        Form_TS_Mode: hlp.localSettingsToView(Settings,activePolicyCache.WebTrafficMonitorPolicy,formModeValues),
        Form_TS_Protect: hlp.localSettingsToView(Settings,activePolicyCache.WebTrafficMonitorPolicy,formProtectValues),
        Form_TS_Web: hlp.localSettingsToView(Settings,activePolicyCache.WebTrafficMonitorPolicy,formWebValues),
        Form_TS_Wc: hlp.localSettingsToView(Settings,activePolicyCache.WebTrafficMonitorPolicy,formWcValues),
        Form_TS_Categorization: hlp.localSettingsToView(Settings,activePolicyCache.WebTrafficMonitorPolicy,formCategorizerValues)
    }
    data.Form_TS_Mode.LocalTask = true;
    // Form_TS_Protect
    data.Form_TS_Protect.ScanSettings = {
        KLPRSS_Val: hlp.localSettingsToView(Settings.Base,activePolicyCache.WebTrafficMonitorPolicy.Base.KLPRSS_Val,formBaseValues)
    };
        //hlp.GetMergedLocalSettingsKey(Settings,activePolicyCache.WebTrafficMonitorPolicy,'Base');
    data.Form_TS_Protect.uiFields = {
        ExtList: data.Form_TS_Protect.ScanSettings.KLPRSS_Val.ScanExtensionMasks.KLPRSS_Val[0].join(),
        ExcludeThreats:data.Form_TS_Protect.ScanSettings.KLPRSS_Val.ExcludeThreats.KLPRSS_Val[0].join(),
        Bigger:(data.Form_TS_Protect.SizeLimit.KLPRSS_Val[0]/(1024*1024))
    };
    data.Form_TS_Protect.LocalTask = true;

    data.Form_TS_Web.UseTz = data.Form_TS_Protect.ScanSettings.KLPRSS_Val.UseTrustedZoneSettings;
    data.Form_TS_Web.LocalTask = true;
    data.Form_TS_Wc.uiFields = utilsTs.toViewWcRules(data.Form_TS_Wc);
    data.Form_TS_Wc.LocalTask = true;
    data.Form_TS_Categorization.uiFields = utilsTs.toViewCategories(data.Form_TS_Categorization);
    data.Form_TS_Categorization.LocalTask = true;
    return data;
}

function toRaw(TASK_ADDITIONAL_PARAMS, changedData, formType) {
    if(formType === 'Schedule' ) {
        hlpSch.toRaw(changedData, TASK_ADDITIONAL_PARAMS.Schedule);
        return;
    }
    else if(formType==='Protection_TS_ControlledPorts') {
        let cpData = {
            UsePortMode: changedData.UsePortMode.KLPRSS_Val[0],
            ControlledPorts: utils.toRawArray(changedData.uiFields.Ports, 'Port')
        };
        Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(cpData, formControlledPortsValues));
        TASK_ADDITIONAL_PARAMS.Settings = utilsTs.FixLocalSettings(TASK_ADDITIONAL_PARAMS.Settings);
        return;
    }
    else if(formType==='Protection_TS_ExcludedPorts'){
        TASK_ADDITIONAL_PARAMS.Settings.ExcludedPorts = utils.toRawArray(changedData.uiFields.Ports, 'Port');
        TASK_ADDITIONAL_PARAMS.Settings = utilsTs.FixLocalSettings(TASK_ADDITIONAL_PARAMS.Settings);
        return;
    }
    else if(formType==='Protection_TS_ExcludedIps'){
        TASK_ADDITIONAL_PARAMS.Settings.ExcludedAddresses = utils.toRawArray(changedData.uiFields.Ips, 'Ip');
        TASK_ADDITIONAL_PARAMS.Settings = utilsTs.FixLocalSettings(TASK_ADDITIONAL_PARAMS.Settings);
        return;
    }
    else if(formType==='Protection_TS_ExcludedProcs'){
        let appData = {
            UseExcludeProcesses:changedData.UseExcludeProcesses.KLPRSS_Val[0],
            ExcludeProcesses: utils.toRawArray(changedData.uiFields.Apps, 'App')
        };
        Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(appData, formExcAppsValues));
        TASK_ADDITIONAL_PARAMS.Settings = utilsTs.FixLocalSettings(TASK_ADDITIONAL_PARAMS.Settings);
        return;
    }

    if(formType!=='Main')
        return;
    // mode
    const Mode = hlp.ItemToLocalSettings(changedData.Form_TS_Mode);
    Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(Mode, formModeValues));
    // protection
    let Protect = hlp.ItemToLocalSettings(changedData.Form_TS_Protect);
    Protect.SizeLimit = Protect.uiFields.Bigger*(1024*1024);
    Protect.ScanSettings.ScanExtensionMasks = utils.normalizedArrayMasks(Protect.uiFields.ExtList);
    Protect.ScanSettings.ExcludeThreats = utils.normalizedArrayMasks(Protect.uiFields.ExcludeThreats);
    Protect.ScanSettings.UseTrustedZoneSettings = changedData.Form_TS_Web.UseTz.KLPRSS_Val[0];
    Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(Protect, formProtectValues));
    Object.assign(TASK_ADDITIONAL_PARAMS.Settings.Base, Protect.ScanSettings);
    //Web
    const Web = hlp.ItemToLocalSettings(changedData.Form_TS_Web);
    Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(Web, formWebValues));
    // Wc
    const Wc = utilsTs.toRowLocalWcRules(changedData.Form_TS_Wc);
    Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(Wc, formWcValues));
    // Categorization
    const Categorization = utilsTs.toRowLocalCategories(changedData.Form_TS_Categorization);
    Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(Categorization, formCategorizerValues));
}

function RbacToPolicyCache(activePolicyCache){
    const allFields = formModeValues.concat(formProtectValues).concat(formWebValues).concat(formWcValues).concat(formCategorizerValues).concat(formBaseValues).concat(formControlledPortsValues);
//    const formExcAppsValues = ['UseExcludeProcesses','ExcludeProcesses'];
    activePolicyCache.WebTrafficMonitorPolicy = hlpPolicy.SetPolicyFieldsRbacLocked(activePolicyCache.WebTrafficMonitorPolicy, allFields);
    activePolicyCache.PredefinedTasksPolicy = hlpPolicy.SetPolicyFieldsRbacLocked(activePolicyCache.PredefinedTasksPolicy, ['TrafficMonitorStartOptions']);
    return activePolicyCache;
}

module.exports = {
    defPolicySections,
    toView,
    toRaw,
    RbacToPolicyCache
};
