const lo = require('lodash');
const hlp = require('../../../../helpers')
const hlpSchedule = require('../Schedule/schedule.js')

const sections = ['AntiCryptorPolicy','PredefinedTasksPolicy']
const formCommonValues = ['EnforcementMode','UseHeuristicLevel','HeuristicLevel','Heuristic']
const formScopeValues = ['Mode','SpecificFolders']
const formExcludesValues = ['UseExcludeFolders','ExcludeFolders']

async function toView(policiesCache) {
    let scope = hlp.customToView(policiesCache.AntiCryptorPolicy.Policy.KLPRSS_Val, formScopeValues);
    scope.UiFolders = scope.SpecificFolders.KLPRSS_Val[0].map(function(node) { return {"Name":node}; })

    let excl = hlp.customToView(policiesCache.AntiCryptorPolicy.Policy.KLPRSS_Val, formExcludesValues);
    excl.UiFolders = excl.ExcludeFolders.KLPRSS_Val[0].map(function(node) { return {"Name":node}; })

    let data = {
        Form_Network_AntiCryptorCommon: hlp.customToView(policiesCache.AntiCryptorPolicy.Policy.KLPRSS_Val, formCommonValues),
        Form_Network_AntiCryptorScope:scope,
        Form_Network_AntiCryptorExcludes:excl,
        Form_Policy_AntiCryptSchedule:hlpSchedule.toView(policiesCache.PredefinedTasksPolicy.Policy.KLPRSS_Val.AntiCryptorStartOptions)
    }
    return data
}

function toRaw(changedData, policiesCache) {
    Object.assign( policiesCache.AntiCryptorPolicy.Policy.KLPRSS_Val, lo.pick(changedData.Form_Network_AntiCryptorCommon, formCommonValues));

    changedData.Form_Network_AntiCryptorScope.SpecificFolders.KLPRSS_Val[0] = changedData.Form_Network_AntiCryptorScope.UiFolders.map(function(node){ return node.Name })
    delete changedData.Form_Network_AntiCryptorScope.UiFolders;
    Object.assign( policiesCache.AntiCryptorPolicy.Policy.KLPRSS_Val, lo.pick(changedData.Form_Network_AntiCryptorScope, formScopeValues));

    changedData.Form_Network_AntiCryptorExcludes.ExcludeFolders.KLPRSS_Val[0] = changedData.Form_Network_AntiCryptorExcludes.UiFolders.map(function(node) { return node.Name })
    delete changedData.Form_Network_AntiCryptorExcludes.UiFolders
    Object.assign( policiesCache.AntiCryptorPolicy.Policy.KLPRSS_Val, lo.pick(changedData.Form_Network_AntiCryptorExcludes, formExcludesValues))

    hlpSchedule.toRaw(changedData.Form_Policy_AntiCryptSchedule, policiesCache.PredefinedTasksPolicy.Policy.KLPRSS_Val.AntiCryptorStartOptions);
}

module.exports = {
    sections,
    toView,
    toRaw
};
