//const { plugin: { pluginScope } } = require('@kl/constants');
//const lo = require('lodash');
const hlp = require('../../helpers');
const hlpPolicy = require('../../helpers_policy');
const consts = require('../../consts');
const profileDef = require('../../default/profile.js');
const profileConverters = require('./sections/index.js');

const tmpDataStoreName = 'profile_cache';

const InitProfilesCache = () => {
    return {policyId: -1, profileId:'', changed:[]};
}

const GetDefaultProfileData = (section) =>{
    if(section === 'TrustedZonePolicy') {
        return profileDef.defaultTrustedZone();
    }
    return {};
}

function PolicyProfile () {
   let profilesCache = InitProfilesCache();

   const CacheFromTempDataStore = async (tempDataStore) => {
        if(tempDataStore===undefined)
            return;
        const tmpCache = await tempDataStore.get(tmpDataStoreName);
        if(tmpCache !== undefined && tmpCache !== {} ){
            profilesCache = tmpCache;
        }
    }

    const CacheToTempDataStore = async (tempDataStore) =>{
        if(tempDataStore!==undefined) {
            await tempDataStore.set(tmpDataStoreName, profilesCache);
        }
    }

    const AddToChangesList = (arrSections) =>{
        if(arrSections===undefined)
            return;
        arrSections = Array.isArray(arrSections) ? arrSections : [arrSections];
        for (let policySection of arrSections) {
            if (policySection!=='' && !profilesCache.changed.includes(policySection)) {
                profilesCache.changed.push(policySection);
            }
        }
    }

    const UpdateRbac = async (api, id, rowData, sectionId) => {
        let policyData = hlp.serialize(rowData);
        try {
            const readOnlyNodes = await api.HstAccessControl.GetPolicyReadonlyNodes({
                IGroupId: id,
                szwProduct: consts.wstrProduct,
                szwVersion: consts.wstrVersion,
                szwSectionName: sectionId,
                pPolicySection: policyData.Policy!==undefined ? policyData.Policy.value : policyData,
            });

            if (readOnlyNodes.result!==undefined ) {
                let changingData = rowData.Policy!==undefined ? rowData.Policy.KLPRSS_Val : rowData;
                hlpPolicy.SetPolicyFieldsRbacLocked(changingData, readOnlyNodes.result[sectionId]);
            }
        }
        catch (error) {
            runtime.logger.error(consts.logProductPref + ' Error: ' + error);
        }

        return rowData;
    }

    const UpdateCache = async (api, profileId, policyId, arrSections) => {
        arrSections = Array.isArray(arrSections) ? arrSections : [arrSections];

        let { result: identifier } = await api.PolicyProfiles.GetProfileSettings({
            nPolicy: policyId, nRevision: 0, szwName: profileId});

        for (const policySection of arrSections) {
            if( profilesCache[policySection] !== undefined ){
                continue;
            }

            const SsPolicySection = consts.policySsSections[policySection];
            if(SsPolicySection === undefined)
                continue;

            const sectionData = await api.SsContents.Ss_Read({
                wstrID: identifier,
                wstrProduct: consts.wstrProduct,
                wstrVersion: consts.wstrVersion,
                wstrSection: SsPolicySection,
                skipDeserialization: true});

            const hasData = (sectionData && sectionData.result && sectionData.result.PxgError===undefined) ? true : false;
            let rowData = hasData ? hlp.deserialize(sectionData.result).KLPRSS_Val : GetDefaultProfileData(policySection);
            rowData = await UpdateRbac (api, policyId, rowData, SsPolicySection);
            profilesCache[policySection] = rowData;
            runtime.logger.log(consts.logProductPref + ' Cached Profile: ' + policySection + ' Data:' + JSON.stringify(rowData));
        }
        api.SsContents.Ss_Release({ wstrID: identifier });
    }

    const SaveChangedProfileSections = async (api, policyId, profileId) => {
        if(profilesCache.changed.length === 0 || profileId === undefined || profileId === '')
            return;

        let { result: identifier } = await api.PolicyProfiles.GetProfileSettings({
            nPolicy: policyId, nRevision: 0, szwName: profileId});
        for (let policySection of profilesCache.changed) {
            const SsPolicySection = consts.policySsSections[policySection];
            let DataToSave = { KLPRSS_Val: profilesCache[policySection] };
            const result = {
                KLPRSS_Mnd: false,
                ...hlp.serialize(DataToSave),
            };

            await api.SsContents.Ss_Replace({
                wstrID: identifier,
                wstrProduct: consts.wstrProduct,
                wstrVersion: consts.wstrVersion,
                wstrSection: SsPolicySection,
                pNewData: result,
                skipSerialization: true
            });
            await api.SsContents.Ss_Apply({ wstrID: identifier });
        }
        api.SsContents.Ss_Release({ wstrID: identifier });
    }

    /*
     * Called via api.js in getFormModel
     */
    const getData = async (api, profileId, policyId, section, tDataStore) => {
        if(profilesCache.policyId !== policyId || profilesCache.profileId !== profileId) {
            profilesCache = InitProfilesCache();
            await CacheToTempDataStore(tDataStore);
        }

        if (section === undefined || profileConverters[section] === undefined)
            return {};

        await CacheFromTempDataStore(tDataStore);
        if (profileConverters[section].sections !== undefined) {
            await UpdateCache(api, profileId, policyId, profileConverters[section].sections);
        }
        await CacheToTempDataStore(tDataStore);

        if (profileConverters[section].toViewEx !== undefined) {
            try {
                return await profileConverters[section].toViewEx(api, profilesCache);
            } catch(error){
                runtime.logger.error(consts.logProductPref + ' toViewEx: ' + error);
                return {};
            }
        }

        if (profileConverters[section].toView !== undefined) {
            try {
                const computedData = profileConverters[section].toView(profilesCache);
                runtime.logger.log(consts.logProductPref + ' getData. Data: ' + JSON.stringify(computedData));
                return computedData;
            } catch(error){
                runtime.logger.error(consts.logProductPref + ' toView: ' + error);
                return {};
            }
        }

        return {};
    }
    /*
     * Called via api.js in setFormModel
     */
    async function updateData (kscApi, section, data, tDataStore) {
        if (section === undefined || profileConverters[section] === undefined)
            return;

        await CacheFromTempDataStore(tDataStore);
        AddToChangesList(profileConverters[section].sections);
        if(profileConverters[section].toRawEx!==undefined)
            await profileConverters[section].toRawEx(kscApi, data, profilesCache);
        else
            profileConverters[section].toRaw(data, profilesCache);
        await CacheToTempDataStore(tDataStore);
    }
    /*
     * Called via api.js in beforeApplyChanges
     */
    function commitData (api, scope) {
    }
    /*
     * Called via api.js in applyChanges
     */
    async function saveData (api, profileId, policyId, tDataStore) {
        await SaveChangedProfileSections(api, policyId, profileId);
        profilesCache = InitProfilesCache();
        await CacheToTempDataStore(tDataStore);
    }

    return {
        getData,
        updateData,
        commitData,
        saveData
    }
}

module.exports = PolicyProfile;
