const hlp = require('../../helpers');
const hlpPolicy = require('../../helpers_policy');
const consts = require('../../consts');
const settConverters = require('./sections/index.js');
const settCommonData = require('./consts.js');

function Settings () {
    let settingsCache = {hostId:'',changed:[]};
    let policiesCache = {};

    async function CacheFromTempDataStore(tempDataStore) {
        if(tempDataStore!==undefined) {
            settingsCache = await tempDataStore.get('local_sett_cache');
            if(settingsCache===undefined || settingsCache==={})
                settingsCache = {hostId:'',changed: []};
        }
    }

    async function CacheToTempDataStore(tempDataStore) {
        if(tempDataStore!==undefined) {
            await tempDataStore.set('local_sett_cache', settingsCache);
        }
    }

    async function PolicyCacheToTempDataStore(tempDataStore) {
        if(tempDataStore!==undefined) {
            await tempDataStore.set('local_sett_policy_cache', policiesCache);
        }
    }

    function AddToChangesList (arrSections) {
        arrSections = Array.isArray(arrSections) ? arrSections : [arrSections];
        for (let section of arrSections) {
            if (section!=='' && !settingsCache.changed.includes(section)) {
                settingsCache.changed.push(section);
            }
        }
    }
    /*
     * Метод добавляет в cache политик новые секции
     */
    async function UpdateSettingsCache (api, arrSections, hostId) {
        arrSections = Array.isArray(arrSections) ? arrSections : [arrSections];
        for (let section of arrSections) {
            if( consts.settSsSections[section]=== undefined || settingsCache[section]!==undefined)
                continue;
            let sectionData = {};
            try{
              sectionData = await api.HostGroup.SS_Read({
                                strHostName: hostId,
                                strType: 'SS_SETTINGS',
                                strProduct: consts.wstrProduct,
                                strVersion: consts.wstrVersion,
                                strSection: consts.settSsSections[section] });
            }
            catch (error) {
                console.error(error)
            }
            const hasData = (sectionData && sectionData.result && sectionData.result.PxgError===undefined) ? true : false;
            const settData = hasData ? sectionData.result : settCommonData.Defaults[section];
            settingsCache[section] = settData;
        }
    }

    async function UpdatePolicyRbacCache (api, arrSections, groupId) {
        arrSections = Array.isArray(arrSections) ? arrSections : [arrSections];
        for (let section of arrSections) {
            if( consts.policySsSections[section]=== undefined || policiesCache[section]===undefined)
                continue;
            const policyData = hlp.serialize(policiesCache[section]);
            let ReadonlyNodes = {};
            const ssPolicySection = consts.policySsSections[section];
            try{
                ReadonlyNodes = await api.HstAccessControl.GetSettingsReadonlyNodes({
                    lGroupId: groupId,
                    szwProduct: consts.wstrProduct,
                    szwVersion: consts.wstrVersion,
                    szwSectionName: ssPolicySection,
                    pSettingsSection:policyData
                });
                if (ReadonlyNodes.result !== undefined) {
                    policiesCache[section] = hlpPolicy.SetPolicyFieldsRbacLocked(policiesCache[section], ReadonlyNodes.result[ssPolicySection]);
                }
            }
            catch (error) {
                console.error(error);
            }
        }
    }

    /*
     * Called via api.js in getFormModel
     */
    async function getSettingsData (api, section, hostId, sGroupId, tempDataStore) {
        const groupId = parseInt(sGroupId);
        if(settingsCache.hostId !== hostId) {
            settingsCache = {hostId:hostId, changed: []};
            await CacheToTempDataStore(tempDataStore);
            policiesCache = {};
            await PolicyCacheToTempDataStore(tempDataStore);
        }
        const activePolicyId = await hlpPolicy.GetHostActivePolicyId(api, hostId);

        if (section === undefined || settConverters[section] === undefined)
            return {};

        if (settConverters[section].sections !== undefined) {
            await UpdateSettingsCache(api, settConverters[section].sections, hostId);
            await CacheToTempDataStore(tempDataStore);

            if(activePolicyId===-1){
                hlpPolicy.UpdatePoliciesCacheAsDefault(settConverters[section].sections, policiesCache, settCommonData.PolicyMappper);
            } else {
                await hlpPolicy.UpdatePoliciesCacheEx(api, activePolicyId, settConverters[section].sections, policiesCache, settCommonData.PolicyMappper);
            }
            await UpdatePolicyRbacCache (api, settConverters[section].sections, groupId);
            await PolicyCacheToTempDataStore(tempDataStore);
        }

        return (settConverters[section].toView !== undefined)  ? settConverters[section].toView(settingsCache,policiesCache) : {}
    }
    /*
     * Called via api.js in setFormModel
     */
    async function updateSettingsData (kscApi, section, data, tempDataStore) {
        if (section === undefined || settConverters[section] === undefined)
            return;
        if (settConverters[section].sections !== undefined) {
            AddToChangesList(settConverters[section].sections);
        }
        try {
            if(settConverters[section].toRawEx!==undefined)
                await settConverters[section].toRawEx(kscApi, data, settingsCache);
            else
                settConverters[section].toRaw(data, settingsCache);
            await CacheToTempDataStore(tempDataStore);
        } catch (error) {
            console.error(error);
        }
    }

    /*
     * Called via api.js in beforeApplyChanges
     */
    function commitSettingsData (api) {
    }

    /*
     * Called via api.js in applyChanges
     */
    async function saveSettingsData (api, hostId, tempDataStore) {
        if(settingsCache.changed.length === 0 )
            return;
        if(!hostId)
            hostId = settingsCache.hostId;
        await CacheFromTempDataStore(tempDataStore);

        for (let section of settingsCache.changed) {
            try{
                await api.HostGroup.SS_Write({
                    strHostName: hostId,
                    strType: 'SS_SETTINGS',
                    strProduct: consts.wstrProduct,
                    strVersion: consts.wstrVersion,
                    strSection: consts.settSsSections[section],
                    nOption: 3,
                    pSettings: settingsCache[section]
                });
            }
            catch (error) {
                console.error(error);
            }
        }

        settingsCache = {hostId:'',changed:[]};
        await CacheToTempDataStore(tempDataStore);
    }

    return {
        getSettingsData,
        updateSettingsData,
        commitSettingsData,
        saveSettingsData
    }
}

module.exports = Settings;
