const { helpers } = require('@kl/openapi-client');
const lo = require('lodash');
const hlpPolicy = require('../../helpers_policy');
const apiUtils = require('../../common/open_api');

const TaskTypeSettingsValue = 'PluginTaskType'
const taskConverters = {
    AutoRuleGenerator: require('./types/AutoRuleGenerator.js'),
    CheckAppIntegrity: require('./types/CheckAppIntegrity.js'),
    CheckFilesIntegrity: require('./types/CheckFilesIntegrity.js'),
    InstallKey: require('./types/InstallKey.js'),
    DeviceRuleGenerator:require('./types/DeviceRuleGenerator.js'),
    OnDemand:require('./types/OnDemand.js'),
	RetranslateUpdates: require('./types/RetranslateUpdates.js'),
    UpdateBases: require('./types/UpdateBases.js'),
	UpdateComponents: require('./types/UpdateComponents.js'),
}

function taskInfo() {
    let taskAdditionalParams = { } ;//initial data
    let currentTaskType = '';
    let curTaskId = '';

    async function CacheFromTempDataStore(tempDataStore) {
        if(tempDataStore!==undefined) {
            taskAdditionalParams = await tempDataStore.get('group_task_cache');
            if(taskAdditionalParams===undefined)
                taskAdditionalParams = {};
        }
    }

    async function CacheToTempDataStore(tempDataStore) {
        if(tempDataStore!==undefined) {
            await tempDataStore.set('group_task_cache', taskAdditionalParams);
        }
    }

    async function getTaskDataCache (kscApi, taskId) {
        if( curTaskId === taskId || taskId === '')
            return
        let result = await hlpPolicy.GetTaskData(kscApi, taskId);
        if ( result && result.TASK_ADDITIONAL_PARAMS !==undefined ) {
            taskAdditionalParams = lo.clone(result.TASK_ADDITIONAL_PARAMS);
            currentTaskType = result.TASK_NAME;
            curTaskId = taskId.toString();
        }
        else{
            curTaskId = ''
            currentTaskType = ''
            taskAdditionalParams = {}
        }
    }

    function mapTaskData(formType,bIsHosted) {
        let computedData = {};
        let converter = taskConverters[currentTaskType];
        if (converter) {
            computedData = converter.toView(taskAdditionalParams, formType,bIsHosted);
        }
        computedData[TaskTypeSettingsValue] = currentTaskType;
        return computedData;
    }

    async function getTaskData ({kscApi, taskId, hostId, formType, tDataStore}) {
        console.log('ksws_windows:> getTaskData ' + JSON.stringify(arguments));
        await getTaskDataCache (kscApi, taskId);
        await CacheToTempDataStore(tDataStore);
        const bIsHosted = await apiUtils.IsSetHostedMode(kscApi);
        return mapTaskData(formType,bIsHosted);
    }

    async function getTaskAdditionalParams(tDataStore) {
        console.log('ksws_windows:GroupTask:getTaskAdditionalParams ' + JSON.stringify(taskAdditionalParams));
        await CacheFromTempDataStore(tDataStore);
        return taskAdditionalParams;
    }

    async function updateTaskData ({kscApi, modelData, formType, tDataStore}) {
        let converter = taskConverters[currentTaskType];
        if (converter) {
            await CacheFromTempDataStore(tDataStore);
            converter.toRaw(taskAdditionalParams, modelData, formType);
            await CacheToTempDataStore(tDataStore);
        }
    }

    async function getWizardTaskData({kscApi, taskType, formType, defData, tDataStore}) {
        let tmpData = {PxgRetVal:lo.clone(defData)};
        let Data = helpers.serializer.deserializeKscRequest({body: tmpData}).result;
        if (!Data || !Data.TASK_ADDITIONAL_PARAMS) {
            return {};
        }

        taskAdditionalParams = lo.clone(Data.TASK_ADDITIONAL_PARAMS);
        await CacheToTempDataStore(tDataStore);
        currentTaskType = taskType;
        const bIsHosted = await apiUtils.IsSetHostedMode(kscApi);
        return mapTaskData(formType,bIsHosted);
    }

    return {
        getTaskData,
        getTaskAdditionalParams,
        updateTaskData,
        getWizardTaskData
    }
}

module.exports = taskInfo
