const taskDefs = require('../../default/tasks.js');
const utils = require('../../utils.js');

const ICAPActionEnum = {Cure:1, Remove:2, Recommended:3, Report:4, Quarantine:5,  Skip:7, Info:8 }
const InfAA  = {disinfect:0, recommended:1, report:2 }
const SuspAA = {quarantine:0,recommended:1, report:2 }

function getIcapInfectedAction(action) {
    switch(action)  {
        case ICAPActionEnum.Skip:
        case ICAPActionEnum.Report:
            return InfAA.report
        case ICAPActionEnum.Info:
        case ICAPActionEnum.Recommended:
            return InfAA.recommended
    }
    return InfAA.disinfect
}

function getIcapSuspAction(action) {
    switch(action)  {
        case ICAPActionEnum.Skip:
        case ICAPActionEnum.Report:
            return SuspAA.report
        case ICAPActionEnum.Info:
        case ICAPActionEnum.Recommended:
            return SuspAA.recommended
    }
    return SuspAA.quarantine
}

function getIcapProtLevelUiFields(scanSetts) {
    return  {
        ExtList: scanSetts.KLPRSS_Val.ScanExtensionMasks.KLPRSS_Val.join(),
        ExcludeMasks: scanSetts.KLPRSS_Val.ExcludeMasks.KLPRSS_Val.join(),
        ExcludeThreats: scanSetts.KLPRSS_Val.ExcludeThreats.KLPRSS_Val.join(),
        InfectedAction: getIcapInfectedAction(scanSetts.KLPRSS_Val.InfectedFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val),
        SuspAction: getIcapSuspAction(scanSetts.KLPRSS_Val.SuspiciousFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val),
        Bigger: (scanSetts.KLPRSS_Val.SizeLimit.KLPRSS_Val/(1024*1024))
    }
}

function IcapScanSettingsFromUiFields(data) {
    if(data.ScanSettings.KLPRSS_Val.ScanType.KLPRSS_Val===4){
        data.ScanSettings.KLPRSS_Val.ScanExtensionMasks.KLPRSS_Val = utils.normalizedArrayExts(data.uiFields.ExtList)
    }
    if(data.ScanSettings.KLPRSS_Val.UseExcludeMasks.KLPRSS_Val===true) {
        data.ScanSettings.KLPRSS_Val.ExcludeMasks.KLPRSS_Val = utils.normalizedArrayMasks(data.uiFields.ExcludeMasks)
    }
    if(data.ScanSettings.KLPRSS_Val.UseExcludeThreats.KLPRSS_Val===true) {
        data.ScanSettings.KLPRSS_Val.ExcludeThreats.KLPRSS_Val = utils.normalizedArrayMasks(data.uiFields.ExcludeThreats)
    }
    if(data.ScanSettings.KLPRSS_Val.UseSizeLimit.KLPRSS_Val===true) {
        data.ScanSettings.KLPRSS_Val.SizeLimit.KLPRSS_Val = data.uiFields.Bigger*1024*1024
    }

    switch (data.uiFields.SuspAction) {
        case InfAA.disinfect:
            data.ScanSettings.KLPRSS_Val.InfectedFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val = ICAPActionEnum.Cure;
            data.ScanSettings.KLPRSS_Val.InfectedSecondAction.KLPRSS_Val.ActionType.KLPRSS_Val = ICAPActionEnum.Report;
            break;
        case InfAA.recommended:
            data.ScanSettings.KLPRSS_Val.InfectedFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val = ICAPActionEnum.Recommended;
            data.ScanSettings.KLPRSS_Val.InfectedSecondAction.KLPRSS_Val.ActionType.KLPRSS_Val = ICAPActionEnum.Recommended;
            break;
        case InfAA.report:
            data.ScanSettings.KLPRSS_Val.InfectedFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val = ICAPActionEnum.Report;
            data.ScanSettings.KLPRSS_Val.InfectedSecondAction.KLPRSS_Val.ActionType.KLPRSS_Val = ICAPActionEnum.Report;
            break;
    }

    switch (data.uiFields.InfectedAction) {
        case SuspAA.quarantine:
            data.ScanSettings.KLPRSS_Val.SuspiciousFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val = ICAPActionEnum.Quarantine;
            data.ScanSettings.KLPRSS_Val.SuspiciousSecondAction.KLPRSS_Val.ActionType.KLPRSS_Val = ICAPActionEnum.Report;
            break;
        case SuspAA.recommended:
            data.ScanSettings.KLPRSS_Val.SuspiciousFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val = ICAPActionEnum.Recommended;
            data.ScanSettings.KLPRSS_Val.SuspiciousSecondAction.KLPRSS_Val.ActionType.KLPRSS_Val = ICAPActionEnum.Recommended;
            break;
        case SuspAA.report:
            data.ScanSettings.KLPRSS_Val.SuspiciousFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val = ICAPActionEnum.Report;
            data.ScanSettings.KLPRSS_Val.SuspiciousSecondAction.KLPRSS_Val.ActionType.KLPRSS_Val = ICAPActionEnum.Report;
            break;
    }

    return data.ScanSettings;
}

function IcapLocalScanSettingsFromUiFields(data) {
    let scanSett = taskDefs.defaultIcapScanScope();
    scanSett.ScanType = data.ScanSettings.KLPRSS_Val.ScanType.KLPRSS_Val;
    scanSett.ScanExtensionMasks = (scanSett.ScanType===4) ? utils.normalizedArrayExts(data.uiFields.ExtList) : [];
    scanSett.UseExcludeMasks = data.ScanSettings.KLPRSS_Val.UseExcludeMasks.KLPRSS_Val;
    scanSett.ExcludeMasks = (scanSett.UseExcludeMasks===true) ? utils.normalizedArrayMasks(data.uiFields.ExcludeMasks) : [];
    scanSett.UseExcludeThreats = data.ScanSettings.KLPRSS_Val.UseExcludeThreats.KLPRSS_Val;
    scanSett.ExcludeThreats = (scanSett.UseExcludeThreats===true) ? utils.normalizedArrayMasks(data.uiFields.ExcludeThreats) : [];
    scanSett.UseSizeLimit = data.ScanSettings.KLPRSS_Val.UseSizeLimit.KLPRSS_Val;
    if(scanSett.UseSizeLimit===true) {
        scanSett.SizeLimit = data.uiFields.Bigger*1024*1024;
    }
    scanSett.UseTimeLimit = data.ScanSettings.KLPRSS_Val.UseTimeLimit.KLPRSS_Val;
    scanSett.TimeLimit = data.ScanSettings.KLPRSS_Val.TimeLimit.KLPRSS_Val;
    scanSett.ScanArchived = data.ScanSettings.KLPRSS_Val.ScanArchived.KLPRSS_Val;
    scanSett.ScanMailBases = data.ScanSettings.KLPRSS_Val.ScanMailBases.KLPRSS_Val;
    scanSett.ScanOle = data.ScanSettings.KLPRSS_Val.ScanOle.KLPRSS_Val;
    scanSett.ScanPacked = data.ScanSettings.KLPRSS_Val.ScanPacked.KLPRSS_Val;
    scanSett.ScanPlainMail = data.ScanSettings.KLPRSS_Val.ScanPlainMail.KLPRSS_Val;
    scanSett.ScanSfxArchived = data.ScanSettings.KLPRSS_Val.ScanSfxArchived.KLPRSS_Val;

    switch (data.uiFields.SuspAction) {
        case InfAA.disinfect:
            scanSett.InfectedFirstAction.ActionType = ICAPActionEnum.Cure;
            scanSett.InfectedSecondAction.ActionType = ICAPActionEnum.Report;
            break;
        case InfAA.recommended:
            scanSett.InfectedFirstAction.ActionType = ICAPActionEnum.Recommended;
            scanSett.InfectedSecondAction.ActionType = ICAPActionEnum.Recommended;
            break;
        case InfAA.report:
            scanSett.InfectedFirstAction.ActionType = ICAPActionEnum.Report;
            scanSett.InfectedSecondAction.ActionType = ICAPActionEnum.Report;
            break;
    }

    switch (data.uiFields.InfectedAction) {
        case SuspAA.quarantine:
            scanSett.SuspiciousFirstAction.ActionType = ICAPActionEnum.Quarantine;
            scanSett.SuspiciousSecondAction.ActionType = ICAPActionEnum.Report;
            break;
        case SuspAA.recommended:
            scanSett.SuspiciousFirstAction.ActionType = ICAPActionEnum.Recommended;
            scanSett.SuspiciousSecondAction.ActionType = ICAPActionEnum.Recommended;
            break;
        case SuspAA.report:
            scanSett.SuspiciousFirstAction.ActionType = ICAPActionEnum.Report;
            scanSett.SuspiciousSecondAction.ActionType = ICAPActionEnum.Report;
            break;
    }

    return {
        AreaDesc:'',
        AreaName:'',
        AreaType:0,
        ScanSettings: scanSett,
        __VersionInfo: [1, 0]
    };
}

module.exports = {
    getIcapProtLevelUiFields,
    IcapScanSettingsFromUiFields,
    IcapLocalScanSettingsFromUiFields
}