const taskDefs = require('../../default/tasks.js');
const utils = require('../../utils.js');
const buffer = require('buffer');
const { TextDecoder, TextEncoder } =  require('util');

function getPswdrData(pswrd) {
    let bytes = Buffer.from(pswrd + String.fromCharCode(0), 'utf-8');
    let bufStr16 = buffer.transcode(bytes, 'utf8', 'utf16le');
    return bufStr16.toString('base64');
}

function toViewAuth(data) {
    const curPriveledgedPswrd = data.PriveledgedAccess.KLPRSS_Val[0].Password.KLPRSS_Val.value;
    data.NewPriveledgedPswrd = '';
    data.UseEmptyPriveledgedPswrd = (curPriveledgedPswrd==='') ? true : false;
    data.PriveledgedPswrdWasSet = (curPriveledgedPswrd==='') ? false : true;

    const curCDOTPswrd = data.cDOTAccess.KLPRSS_Val[0].Password.KLPRSS_Val.value;
    data.NewCDOTPswrd = '';
    data.UseEmptyCDOTPswrd = (curCDOTPswrd==='') ? true : false;
    data.CDOTPswrdWasSet = (curCDOTPswrd==='') ? false : true;

    data.Items = utils.toViewArray(data.cDOTNodesList.KLPRSS_Val[0], 'Item')
    return data;
}

function toViewExclusions(data) {
    data.ExcludeRules.KLPRSS_Val[0].forEach(function(item) {
        item.Form_Nas_AntiCrypt_ExcludeMasks = {
            Items: utils.toViewArray(item.PathMasks.KLPRSS_Val, 'Item'),
            UsePathMask: item.UsePathMask.KLPRSS_Val
        }

        item.Form_Nas_AntiCrypt_ExcludeIps = {
            Ips: utils.toViewArray(item.IpAddresses.KLPRSS_Val, 'Ip'),
            UseIpAddress: item.UseIpAddress.KLPRSS_Val
        }
        item.Form_Nas_AntiCrypt_ExcludeUsers = {
            Users: utils.toViewArray(item.UserSids.KLPRSS_Val, 'User'),
            UseUserSid: item.UseUserSid.KLPRSS_Val
        }
    });
    return data;
}

async function toRawAuth(changedData, kscApi) {
    // PriveledgedAccess
    changedData.PriveledgedAccess.KLPRSS_Val[0].Enable.KLPRSS_Val = changedData.PriveledgedAccess.KLPRSS_Val[0].Login.KLPRSS_Val==='' ? false : true;

    if(changedData.UseEmptyPriveledgedPswrd){
        changedData.PriveledgedAccess.KLPRSS_Val[0].Password.KLPRSS_Val.value = '';
    }
    else if(changedData.NewPriveledgedPswrd!=='') {
        const pswrd_data = getPswdrData(changedData.NewPriveledgedPswrd);
        const response = await kscApi.DataProtectionApi.ProtectDataGlobally({ pData:pswrd_data })
        if(response.result!=='')
            changedData.PriveledgedAccess.KLPRSS_Val[0].Password.KLPRSS_Val.value = response.result;
    }
    // cDOTAccess
    changedData.cDOTAccess.KLPRSS_Val[0].Enable.KLPRSS_Val = changedData.cDOTAccess.KLPRSS_Val[0].Login.KLPRSS_Val==='' ? false : true;

    if(changedData.UseEmptyCDOTPswrd){
        changedData.cDOTAccess.KLPRSS_Val[0].Password.KLPRSS_Val.value = '';
    }
    else if(changedData.NewCDOTPswrd!=='') {
        const pswrd_data = getPswdrData(changedData.NewCDOTPswrd);
        const response = await kscApi.DataProtectionApi.ProtectDataGlobally({ pData:pswrd_data })
        if(response.result!=='')
            changedData.cDOTAccess.KLPRSS_Val[0].Password.KLPRSS_Val.value = response.result;
    }

    changedData.cDOTNodesList.KLPRSS_Val[0] = utils.toRawArray(changedData.Items, 'Item')
    delete changedData.Items;

    return changedData;
}

function toRowExclusions(changedData) {
    changedData.ExcludeRules.KLPRSS_Val[0].forEach(function(item) {
        item.UsePathMask.KLPRSS_Val = item.Form_Nas_AntiCrypt_ExcludeMasks.UsePathMask
        if(item.Form_Nas_AntiCrypt_ExcludeMasks.UsePathMask)
            item.PathMasks.KLPRSS_Val = utils.toRawArray(item.Form_Nas_AntiCrypt_ExcludeMasks.Items, 'Item')

        item.UseIpAddress.KLPRSS_Val = item.Form_Nas_AntiCrypt_ExcludeIps.UseIpAddress
        if(item.Form_Nas_AntiCrypt_ExcludeIps.UseIpAddress)
            item.IpAddresses.KLPRSS_Val = utils.toRawArray(item.Form_Nas_AntiCrypt_ExcludeIps.Ips, 'Ip')

        item.UseUserSid.KLPRSS_Val = item.Form_Nas_AntiCrypt_ExcludeUsers.UseUserSid
        if(item.Form_Nas_AntiCrypt_ExcludeUsers.UseUserSid)
            item.UserSids.KLPRSS_Val = utils.toRawArray(item.Form_Nas_AntiCrypt_ExcludeUsers.Users, 'User')

        delete item.Form_Nas_AntiCrypt_ExcludeMasks;
        delete item.Form_Nas_AntiCrypt_ExcludeIps;
        delete item.Form_Nas_AntiCrypt_ExcludeUsers;
    });

    return changedData;
}

function toRowSettExclusions(changedData) {
    let exclRules = [];
    changedData.ExcludeRules.KLPRSS_Val[0].forEach(function(item) {
        let exclude = taskDefs.defaultNasacExclude();
        exclude.Enable = item.Enable.KLPRSS_Val;

        exclude.UsePathMask = item.Form_Nas_AntiCrypt_ExcludeMasks.UsePathMask;
        exclude.PathMasks = exclude.UsePathMask ? utils.toRawArray(item.Form_Nas_AntiCrypt_ExcludeMasks.Items, 'Item') : [];

        exclude.UseIpAddress = item.Form_Nas_AntiCrypt_ExcludeIps.UseIpAddress;
        exclude.IpAddresses = exclude.UseIpAddress ? utils.toRawArray(item.Form_Nas_AntiCrypt_ExcludeIps.Ips, 'Ip') : [];

        exclude.UseUserSid = item.Form_Nas_AntiCrypt_ExcludeUsers.UseUserSid;
        exclude.UserSids = exclude.UseUserSid ? utils.toRawArray(item.Form_Nas_AntiCrypt_ExcludeUsers.Users, 'User') : [];

        exclRules.push(exclude);
    });

    return {
        UseExcludes: changedData.UseExcludes.KLPRSS_Val[0],
        ExcludeRules: exclRules
    };
}

async function toRawSettAuth(changedData, kscApi, hostId) {
    // PriveledgedAccess
    changedData.PriveledgedAccess.KLPRSS_Val[0].Enable.KLPRSS_Val = changedData.PriveledgedAccess.KLPRSS_Val[0].Login.KLPRSS_Val==='' ? false : true;

    if(changedData.UseEmptyPriveledgedPswrd){
        changedData.PriveledgedAccess.KLPRSS_Val[0].Password.KLPRSS_Val.value = '';
    }
    else if(changedData.NewPriveledgedPswrd!=='') {
        const pswrd_data = getPswdrData(changedData.NewPriveledgedPswrd);
        const response = await kscApi.DataProtectionApi.ProtectDataForHost ({ szwHostId:hostId, pData:pswrd_data });
        if(response.result!=='')
            changedData.PriveledgedAccess.KLPRSS_Val[0].Password.KLPRSS_Val.value = response.result;
    }
    // cDOTAccess
    changedData.cDOTAccess.KLPRSS_Val[0].Enable.KLPRSS_Val = changedData.cDOTAccess.KLPRSS_Val[0].Login.KLPRSS_Val==='' ? false : true;

    if(changedData.UseEmptyCDOTPswrd){
        changedData.cDOTAccess.KLPRSS_Val[0].Password.KLPRSS_Val.value = '';
    }
    else if(changedData.NewCDOTPswrd!=='') {
        const pswrd_data = getPswdrData(changedData.NewCDOTPswrd);
        const response = await kscApi.DataProtectionApi.ProtectDataForHost ({ szwHostId:hostId, pData:pswrd_data });
        if(response.result!=='')
            changedData.cDOTAccess.KLPRSS_Val[0].Password.KLPRSS_Val.value = response.result;
    }

    changedData.cDOTNodesList.KLPRSS_Val[0] = utils.toRawArray(changedData.Items, 'Item');
    delete changedData.Items;

    return changedData;
}


module.exports = {
    toViewAuth,
    toViewExclusions,
    toRawAuth,
    toRowExclusions,
    toRowSettExclusions,
    toRawSettAuth,
}