//const enums = require('../../types');
const hlpDefPolicy = require('../../default/policy.js');
const utils = require('../../utils');

const OasActionEnum = { Cure:1, Remove:2, Recommended:3, Report:4, Quarantine:5, Skip:7, Info:8 };
const ObjAction = { notify:0, block:1 };
const InfAA = { cure:0, cure_delete:1, delete:2, recommended:3, skip:100 };
const SuspAA = { quarantine: 0, delete:1, recommended:2, skip:100 };

function ItemToLocalSettings(item) {
    if(Array.isArray(item)) {
        return item.map(function(node) { return ItemToLocalSettings(node); });
    }

    Object.keys(item).forEach(property => {
        if(item[property].KLPRSS_Val!==undefined){
            if(property==='__VersionInfo'){
                item[property] = item[property].KLPRSS_Val;
            }
            else{
                let val = item[property].KLPRSS_Val;
                if(Array.isArray(val) && item[property].KLPRSS_Val[0]!== undefined){
                    val = item[property].KLPRSS_Val[0];
                }
                item[property] = (typeof val === 'object') ? ItemToLocalSettings(val) : val;
            }
        }
    })
    return item;
}

const getUIFields = (scanSett, isIncluded) => {
    let infAction = {};
    switch(scanSett.InfectedFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val)  {
        case OasActionEnum.Info:
            infAction = {Action: ObjAction.notify, UseAdd:false, AddAction:InfAA.recommended};
            break;
        case OasActionEnum.Skip:
        case OasActionEnum.Report:
            infAction = {Action: ObjAction.block, UseAdd:false, AddAction:InfAA.recommended};
            break;
        case OasActionEnum.Cure:
            const ia = (scanSett.InfectedSecondAction.KLPRSS_Val.ActionType.KLPRSS_Val === OasActionEnum.Remove ? InfAA.cure_delete : InfAA.cure);
            infAction = {Action: ObjAction.block, UseAdd:true, AddAction:ia};
            break;
        case OasActionEnum.Remove:
            infAction = {Action: ObjAction.block, UseAdd:true, AddAction:InfAA.delete};
            break;
        case OasActionEnum.Recommended:
            infAction = {Action: ObjAction.block, UseAdd:true, AddAction:InfAA.recommended};
            break;
    }

    let suspAction = {};
    switch(scanSett.SuspiciousFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val)  {
        case OasActionEnum.Info:
            suspAction = {Action:ObjAction.notify,UseAdd:false,AddAction:SuspAA.recommended};
            break;
        case OasActionEnum.Skip:
        case OasActionEnum.Report:
            suspAction = {Action:ObjAction.block,UseAdd:false,AddAction:SuspAA.recommended};
            break;
        case OasActionEnum.Quarantine:
            suspAction = {Action:ObjAction.block,UseAdd:true,AddAction:SuspAA.quarantine};
            break;
        case OasActionEnum.Remove:
            suspAction = {Action:ObjAction.block,UseAdd:true,AddAction:SuspAA.delete};
            break;
        case OasActionEnum.Recommended:
            suspAction = {Action:ObjAction.block,UseAdd:true,AddAction:SuspAA.recommended};
            break;
    }
    return {
        Included:isIncluded,
        InfAction:infAction,
        SuspAction: suspAction,
        ExtList: scanSett.ScanExtensionMasks.KLPRSS_Val.join(),
        ExcludeMasks: scanSett.ExcludeMasks.KLPRSS_Val.join(),
        ExcludeThreats: scanSett.ExcludeThreats.KLPRSS_Val.join(),
        Bigger: scanSett.SizeLimit.KLPRSS_Val/(1024*1024)
    }
}

function updateFromUIFields(scanSett, uiFields) {
    if(scanSett.ScanType.KLPRSS_Val===4){
        scanSett.ScanExtensionMasks.KLPRSS_Val = utils.normalizedArrayExts(uiFields.ExtList);
    }
    if(scanSett.UseExcludeMasks.KLPRSS_Val===true) {
        scanSett.ExcludeMasks.KLPRSS_Val = utils.normalizedArrayMasks(uiFields.ExcludeMasks);
    }
    if(scanSett.UseExcludeThreats.KLPRSS_Val===true) {
        scanSett.ExcludeThreats.KLPRSS_Val = utils.normalizedArrayMasks(uiFields.ExcludeThreats)
    }
    if(scanSett.UseSizeLimit.KLPRSS_Val===true) {
        scanSett.SizeLimit.KLPRSS_Val = uiFields.Bigger*(1024*1024);
    }

    // update Infect actions
    if (uiFields.InfAction.Action === ObjAction.notify) {
        scanSett.InfectedFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val  = OasActionEnum.Info
        scanSett.InfectedSecondAction.KLPRSS_Val.ActionType.KLPRSS_Val = OasActionEnum.Info
    }
    else {
        if (uiFields.InfAction.UseAdd===false)
            uiFields.InfAction.AddAction = InfAA.skip

        switch (uiFields.InfAction.AddAction) {
            case InfAA.cure:
                scanSett.InfectedFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val = OasActionEnum.Cure
                scanSett.InfectedSecondAction.KLPRSS_Val.ActionType.KLPRSS_Val = OasActionEnum.Skip
                break;
            case InfAA.cure_delete:
                scanSett.InfectedFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val = OasActionEnum.Cure;
                scanSett.InfectedSecondAction.KLPRSS_Val.ActionType.KLPRSS_Val = OasActionEnum.Remove;
                break;
            case InfAA.delete:
                scanSett.InfectedFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val = OasActionEnum.Remove;
                scanSett.InfectedSecondAction.KLPRSS_Val.ActionType.KLPRSS_Val = OasActionEnum.Skip;
                break;
            case InfAA.recommended:
                scanSett.InfectedFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val = OasActionEnum.Recommended;
                scanSett.InfectedSecondAction.KLPRSS_Val.ActionType.KLPRSS_Val = OasActionEnum.Recommended;
                break;
            default:
                scanSett.InfectedFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val = OasActionEnum.Skip
                scanSett.InfectedSecondAction.KLPRSS_Val.ActionType.KLPRSS_Val = OasActionEnum.Skip
                break;
        }
    }
    // update Susp actions
    if (uiFields.SuspAction.Action === ObjAction.notify) {
        scanSett.SuspiciousFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val = OasActionEnum.Info;
        scanSett.SuspiciousSecondAction.KLPRSS_Val.ActionType.KLPRSS_Val = OasActionEnum.Info;
    }
    else {
        if (uiFields.SuspAction.UseAdd===false)
            uiFields.SuspAction.AddAction = SuspAA.skip;

        switch (uiFields.SuspAction.AddAction) {
            case SuspAA.quarantine:
                scanSett.SuspiciousFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val = OasActionEnum.Quarantine;
                scanSett.SuspiciousSecondAction.KLPRSS_Val.ActionType.KLPRSS_Val = OasActionEnum.Skip;
                break;
            case SuspAA.delete:
                scanSett.SuspiciousFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val = OasActionEnum.Remove;
                scanSett.SuspiciousSecondAction.KLPRSS_Val.ActionType.KLPRSS_Val = OasActionEnum.Skip;
                break;
            case SuspAA.recommended:
                scanSett.SuspiciousFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val = OasActionEnum.Recommended;
                scanSett.SuspiciousSecondAction.KLPRSS_Val.ActionType.KLPRSS_Val = OasActionEnum.Recommended;
                break;
            default:
                scanSett.SuspiciousFirstAction.KLPRSS_Val.ActionType.KLPRSS_Val = OasActionEnum.Skip;
                scanSett.SuspiciousSecondAction.KLPRSS_Val.ActionType.KLPRSS_Val = OasActionEnum.Skip;
                break;
        }
    }
}

function OasScopeSettingsFromUiFields(scanSett, uiFields) {
    scanSett.ScanExtensionMasks = (scanSett.ScanType===4) ? utils.normalizedArrayExts(uiFields.ExtList) : [];
    scanSett.ExcludeMasks = (scanSett.UseExcludeMasks===true) ? utils.normalizedArrayMasks(uiFields.ExcludeMasks) : [];
    scanSett.ExcludeThreats = (scanSett.UseExcludeThreats===true) ? utils.normalizedArrayMasks(uiFields.ExcludeThreats) : [];
    if(scanSett.UseSizeLimit===true) {
        scanSett.SizeLimit = uiFields.Bigger*(1024*1024);
    }

    // update Infect actions
    if (uiFields.InfAction.Action === ObjAction.notify) {
        scanSett.InfectedFirstAction.ActionType  = OasActionEnum.Info
        scanSett.InfectedSecondAction.ActionType = OasActionEnum.Info
    }
    else {
        if (uiFields.InfAction.UseAdd===false)
            uiFields.InfAction.AddAction = InfAA.skip

        switch (uiFields.InfAction.AddAction) {
            case InfAA.cure:
                scanSett.InfectedFirstAction.ActionType = OasActionEnum.Cure
                scanSett.InfectedSecondAction.ActionType = OasActionEnum.Skip
                break;
            case InfAA.cure_delete:
                scanSett.InfectedFirstAction.ActionType = OasActionEnum.Cure;
                scanSett.InfectedSecondAction.ActionType = OasActionEnum.Remove;
                break;
            case InfAA.delete:
                scanSett.InfectedFirstAction.ActionType = OasActionEnum.Remove;
                scanSett.InfectedSecondAction.ActionType = OasActionEnum.Skip;
                break;
            case InfAA.recommended:
                scanSett.InfectedFirstAction.ActionType = OasActionEnum.Recommended;
                scanSett.InfectedSecondAction.ActionType = OasActionEnum.Recommended;
                break;
            default:
                scanSett.InfectedFirstAction.ActionType = OasActionEnum.Skip
                scanSett.InfectedSecondAction.ActionType = OasActionEnum.Skip
                break;
        }
    }
    // update Susp actions
    if (uiFields.SuspAction.Action === ObjAction.notify) {
        scanSett.SuspiciousFirstAction.ActionType = OasActionEnum.Info;
        scanSett.SuspiciousSecondAction.ActionType = OasActionEnum.Info;
    }
    else {
        if (uiFields.SuspAction.UseAdd===false)
            uiFields.SuspAction.AddAction = SuspAA.skip;

        switch (uiFields.SuspAction.AddAction) {
            case SuspAA.quarantine:
                scanSett.SuspiciousFirstAction.ActionType = OasActionEnum.Quarantine;
                scanSett.SuspiciousSecondAction.ActionType = OasActionEnum.Skip;
                break;
            case SuspAA.delete:
                scanSett.SuspiciousFirstAction.ActionType = OasActionEnum.Remove;
                scanSett.SuspiciousSecondAction.ActionType = OasActionEnum.Skip;
                break;
            case SuspAA.recommended:
                scanSett.SuspiciousFirstAction.ActionType = OasActionEnum.Recommended;
                scanSett.SuspiciousSecondAction.ActionType = OasActionEnum.Recommended;
                break;
            default:
                scanSett.SuspiciousFirstAction.ActionType = OasActionEnum.Skip;
                scanSett.SuspiciousSecondAction.ActionType = OasActionEnum.Skip;
                break;
        }
    }
}

const GetOasScanScopeSettings = (item) => {
    const advActions = ItemToLocalSettings(item.ScanSettings.KLPRSS_Val.AdvancedActions.KLPRSS_Val);
    let scope = ItemToLocalSettings(item);
    OasScopeSettingsFromUiFields(scope.ScanSettings, scope.uiFields);
    scope.ScanSettings.AdvancedActions = advActions.map(a=>{
        const Act = {
            Actions: [{
                DetectCertainty: a.Actions.DetectCertainty,
                FirstAction: a.Actions.FirstAction,
                SecondAction: a.Actions.SecondAction,
                __VersionInfo: a.Actions.__VersionInfo
            }],
            Verdict: a.Verdict,
            __VersionInfo: a.__VersionInfo
        };
        return Act;
    });

    delete scope.uiFields;
    return scope;
}

const toViewScanScope = (data) => {
    data.Areas = [];
    for(let scope of data.ScanScope.KLPRSS_Val[0]){
        scope.uiFields = getUIFields(scope.ScanSettings.KLPRSS_Val, true);
        data.Areas.push(scope);
    }

    for(let item of data.ExcludedFromScanScope.KLPRSS_Val[0]){
        let exclude = hlpDefPolicy.defaultOasScope();
        exclude.uiFields.Included = false;
        exclude.AreaName = item.AreaName;
        exclude.AreaType = item.AreaType;
        data.Areas.push(exclude);
    }

    return data;
}

const toRowScanScope = (changedData) => {
    let scanScope = [];
    let scanExcludes = [];
    for(let item of changedData.Areas){
        if(item.uiFields.Included === true){
            let scope = item;
            updateFromUIFields(scope.ScanSettings.KLPRSS_Val, scope.uiFields);
            delete scope.uiFields;
            scanScope.push(scope);
        }
        else{
            scanExcludes.push({
                AreaName:item.AreaName,
                AreaType:item.AreaType,
                __VersionInfo:{KLPRSS_Val:[1,0],KLPRSS_Mnd:false}
            });
        }
    }

    Object.assign(changedData.ScanScope.KLPRSS_Val[0], scanScope);
    Object.assign(changedData.ExcludedFromScanScope.KLPRSS_Val[0], scanExcludes);
    return changedData;
}

const toRowSettingsScanScope = (changedData) => {
    let scanScope = [];
    let scanExcludes = [];
    for(let item of changedData.Areas){
        if(item.uiFields.Included === true){
            const scope = GetOasScanScopeSettings(item);
            scanScope.push(scope);
        }
        else{
            scanExcludes.push({
                AreaName:item.AreaName.KLPRSS_Val,
                AreaType:item.AreaType.KLPRSS_Val,
                __VersionInfo:[1,0]
            });
        }
    }
    return {
        ScanScope: scanScope,
        ExcludedFromScanScope: scanExcludes
    };
}

module.exports = {
    getUIFields,
    toViewScanScope,
    toRowScanScope,
    toRowSettingsScanScope
}