const util = require('../../utils');
const hlpDefPolicy = require('../../default/policy.js');

const emptyXmlTag = '\r\n\t\t\t\t\t';

const toViewExcludes = (rowExcludes) => {
    let excludes = rowExcludes;
    for(let item of excludes.KLPRSS_Val[0]){
        item.MasksStr = item.ThreatNameMasks.KLPRSS_Val.join(";");
        if(item.ApplyForIDS===undefined) {
            item.ApplyForIDS = { KLPRSS_Mnd:false, KLPRSS_Val: false }
        }
    }
    return excludes;
}

const toRowExcludes = (viewExcludes) => {
    for(let item of viewExcludes.KLPRSS_Val[0]) {
        if(item.ThreatNameMasks === undefined)
            item.ThreatNameMasks = {};
        item.ThreatNameMasks.KLPRSS_Val = util.normalizedArrayMasks(item.MasksStr);
        delete item.MasksStr;

        for(let object of item.Objects.KLPRSS_Val) {
            object.__VersionInfo = {KLPRSS_Mnd:false, KLPRSS_Val:[1,1]}
        }
        item.__VersionInfo = {KLPRSS_Mnd:false, KLPRSS_Val:[1,6]}
    }
    return viewExcludes;
}

const toRowLocalSettsExcludes = (viewExcludes) => {
    const excludes = viewExcludes.KLPRSS_Val[0].map((item)=> {
        return {
            ApplyForFileMonitor: item.ApplyForFileMonitor.KLPRSS_Val,
            ApplyForICAPScanner: item.ApplyForICAPScanner.KLPRSS_Val,
            ApplyForMailTrafficMonitor: item.ApplyForMailTrafficMonitor.KLPRSS_Val,
            ApplyForNetAppMonitor: item.ApplyForNetAppMonitor.KLPRSS_Val,
            ApplyForOnDemandScan: item.ApplyForOnDemandScan.KLPRSS_Val,
            ApplyForScriptChecker: item.ApplyForScriptChecker.KLPRSS_Val,
            ApplyForWebTrafficMonitor: item.ApplyForWebTrafficMonitor.KLPRSS_Val,
            ApplyForIDS: item.ApplyForIDS ? item.ApplyForIDS.KLPRSS_Val : false,
            Description: item.Description.KLPRSS_Val,
            Enabled: item.Enabled.KLPRSS_Val,
            Objects: item.Objects.KLPRSS_Val.map((obj)=>{
                return {
                    Path: obj.Path.KLPRSS_Val,
                    Type: obj.Type.KLPRSS_Val,
                    UseInSubfolders: obj.UseInSubfolders.KLPRSS_Val,
                    __VersionInfo:[1,1]
                };
            }),
            ThreatNameMasks: util.normalizedArrayMasks(item.MasksStr),
            UseObjects: item.UseObjects.KLPRSS_Val,
            UseThreatNameMasks: item.UseThreatNameMasks.KLPRSS_Val,
            __VersionInfo: [1,6]
        };
    });
    return excludes;
}

const toViewTrustedProcesses = (rowTrustProcs) => {
    let trustProcs = rowTrustProcs;
    for(let item of trustProcs.KLPRSS_Val[0]){
        item.ModuleHash = '';
        if(item.ExecuteModuleHash!==undefined){
            const hash = item.ExecuteModuleHash.KLPRSS_Val;
            item.ModuleHash = (hash.type==='binary') ? Buffer.from(hash.value, 'base64').toString('hex') : '';
        }
    }
    return trustProcs;
}

const toRowTrustedProcesses = (viewTrustProcs) => {
    for(let item of viewTrustProcs.KLPRSS_Val[0]){
        if(!item.ExecuteModuleHash)
            item.ExecuteModuleHash = {KLPRSS_Val: {type:'binary', value: ''}}

        const base64 = Buffer.from(item.ModuleHash, 'hex').toString('base64');
        item.ExecuteModuleHash.KLPRSS_Val.value = base64;
        item.__VersionInfo = {KLPRSS_Mnd:false, KLPRSS_Val:[1,1]};
        delete item.ModuleHash;
    }
    return viewTrustProcs;
}

const toViewExcludeXmlRule = (xmlRule) => {
    let rule = hlpDefPolicy.defaultTzExclude();
    rule.ApplyForFileMonitor.KLPRSS_Val = xmlRule.ApplyForFileMonitor[0] === 'yes';
    rule.ApplyForOnDemandScan.KLPRSS_Val = xmlRule.ApplyForOnDemandScan[0] === 'yes';
    rule.ApplyForICAPScanner.KLPRSS_Val = xmlRule.ApplyForICAPScanner[0] === 'yes ';
    rule.ApplyForMailTrafficMonitor.KLPRSS_Val = xmlRule.ApplyForMailTrafficMonitor[0] === 'yes';
    rule.ApplyForNetAppMonitor.KLPRSS_Val = xmlRule.ApplyForNetAppMonitor[0] === 'yes';
    rule.ApplyForScriptChecker.KLPRSS_Val = xmlRule.ApplyForScriptChecker[0] === 'yes';
    rule.ApplyForWebTrafficMonitor.KLPRSS_Val = xmlRule.ApplyForWebTrafficMonitor[0] === 'yes';
    rule.ApplyForIDS.KLPRSS_Val = (xmlRule.ApplyForIDS && xmlRule.ApplyForIDS[0] === 'yes');
    rule.Description.KLPRSS_Val = xmlRule.Description[0];
    rule.UseObjects.KLPRSS_Val = xmlRule.UseObjects[0] === 'yes';
    rule.Enabled.KLPRSS_Val = xmlRule.Enabled[0] === 'yes';
    rule.UseThreatNameMasks.KLPRSS_Val = xmlRule.UseThreatNameMasks[0] === 'yes';
    rule.MasksStr = xmlRule.ThreatNameMasks[0].element ? xmlRule.ThreatNameMasks[0].element.join(";") : '';

    rule.Objects.KLPRSS_Val = (!xmlRule.Objects[0].element) ? [] :
        xmlRule.Objects[0].element.map((obj)=>{
            return {
                Path: {KLPRSS_Mnd:false, KLPRSS_Val: obj.Path[0]},
                Type: {KLPRSS_Mnd:false, KLPRSS_Val: parseInt(obj.Type[0])},
                UseInSubfolders: {KLPRSS_Mnd:false, KLPRSS_Val:obj.UseInSubfolders[0] === 'yes'},
                __VersionInfo: {KLPRSS_Mnd:false, KLPRSS_Val:[1,1]}
            };
        });

    return rule;
}

const toViewTrustedProcessXmlRule = (xmlRule) => {
    let rule = hlpDefPolicy.defaultTzTrustedProc();
    rule.CertificateSubject.KLPRSS_Val = xmlRule.CertificateSubject[0].replace(/[\r\n\t]/g, "");
    rule.CertificateThumbprint.KLPRSS_Val = xmlRule.CertificateThumbprint[0].replace(/[\r\n\t]/g, "");
    rule.ChechMethodType.KLPRSS_Val = parseInt(xmlRule.ChechMethodType[0]);
    rule.ExecuteModule.KLPRSS_Val = xmlRule.ExecuteModule[0];
    rule.ExecuteModuleHashType.KLPRSS_Val = parseInt(xmlRule.ExecuteModuleHashType[0]);
    rule.ExecutePath.KLPRSS_Val = xmlRule.ExecutePath[0];
    rule.IsTrustedProcess.KLPRSS_Val = xmlRule.IsTrustedProcess[0] === 'yes';
    rule.UseCertificateSubject.KLPRSS_Val = xmlRule.UseCertificateSubject[0] === 'yes';
    rule.UseCertificateThumbprint.KLPRSS_Val = xmlRule.UseCertificateThumbprint[0] === 'yes';
    rule.ModuleHash = xmlRule.ExecuteModuleHash!==undefined ? xmlRule.ExecuteModuleHash[0].replace(/[\r\n\t]/g, "") : '';
    return rule;
}

const isExclusionsEqual = (e1, e2) => {
    if( e1.UseObjects.KLPRSS_Val !== e2.UseObjects.KLPRSS_Val )
        return false;
    if( e1.ApplyForFileMonitor.KLPRSS_Val !== e2.ApplyForFileMonitor.KLPRSS_Val )
        return false;
    if( e1.ApplyForOnDemandScan.KLPRSS_Val !== e2.ApplyForOnDemandScan.KLPRSS_Val )
        return false;
/*
    if( e1.ApplyForScriptChecker.KLPRSS_Val !== e2.ApplyForScriptChecker.KLPRSS_Val )
        return false;
    if( e1.ApplyForNetAppMonitor.KLPRSS_Val !== e2.ApplyForNetAppMonitor.KLPRSS_Val )
        return false;
    if( e1.ApplyForMailTrafficMonitor.KLPRSS_Val !== e2.ApplyForMailTrafficMonitor.KLPRSS_Val )
        return false;
 */
    // are threats list equal
    if( e1.MasksStr !== e2.MasksStr )
        return false;
    // are objects equal
    if( e1.Objects.KLPRSS_Val.length !== e2.Objects.KLPRSS_Val.length )
        return false;
    const o1 = JSON.stringify(e1.Objects.KLPRSS_Val);
    const o2 = JSON.stringify(e2.Objects.KLPRSS_Val);
    return ( o1 === o2 );
}

const isTrustedProcEqual = (p1, p2) => {
    if (p1.ChechMethodType.KLPRSS_Val!== p2.ChechMethodType.KLPRSS_Val){
        return false;
    }
    // the same rule based on file path
    if (p1.ChechMethodType.KLPRSS_Val === 1 &&
        (p1.ExecuteModule.KLPRSS_Val===p2.ExecuteModule.KLPRSS_Val && p1.ExecutePath.KLPRSS_Val===p2.ExecutePath.KLPRSS_Val)) {
        return true;
    }
    // the same rule based on hash
    if (p1.ChechMethodType.KLPRSS_Val===2 && p1.ModuleHash===p2.ModuleHash){
        return true;
    }
    // the same rule based on file path and hash
    if ( p1.ChechMethodType.KLPRSS_Val===3 &&
        (p1.ExecuteModule.KLPRSS_Val===p2.ExecuteModule.KLPRSS_Val && p1.ExecutePath.KLPRSS_Val===p2.ExecutePath.KLPRSS_Val) &&
        p1.ModuleHash===p2.ModuleHash){
        return true;
    }
    return false;
}

module.exports = {
    toViewExcludes,
    toRowExcludes,
    toRowLocalSettsExcludes,
    toViewTrustedProcesses,
    toRowTrustedProcesses,
    toViewExcludeXmlRule,
    toViewTrustedProcessXmlRule,
    isTrustedProcEqual,
    isExclusionsEqual
}