import {CompoundObjectScanModeEnum, ScanTypeEnum} from "../types";

const OdsActionEnum = {Cure: 1, Remove: 2, Recommended: 3, Report: 4, Quarantine: 5, Skip: 7, Postpone: 8};
const HSMScanActionEnum = { ScanEntireFile: 1, ScanResidentPortion: 2, Skip: 3};

const CheckOdsLevel = (data) => {
    if( data.ScanObjects === true &&
        data.ScanType === ScanTypeEnum.AllObjects &&
        data.ScanSectors === true &&
        data.ScanStreams === true &&
        data.ScanArchived === true &&
        data.ScanSfxArchived === true &&
        data.ScanMailBases === true &&
        data.ScanPlainMail === true &&
        data.ScanPacked === true &&
        data.ScanOle === true &&
        data.UseTimeLimit === false &&
        data.UseSizeLimit === false &&
        data.InfectedFirstAction.ActionType === OdsActionEnum.Cure &&
        data.InfectedSecondAction.ActionType === OdsActionEnum.Remove &&
        data.SuspiciousFirstAction.ActionType === OdsActionEnum.Quarantine &&
        data.SuspiciousSecondAction.ActionType === OdsActionEnum.Report &&
        data.UseAdvancedActions === false &&
        data.UseExcludeMasks === false &&
        data.UseExcludeThreats === false &&
        data.ScanOnlyNewAndChangedFiles === false &&
        data.ScanArchivedMode === CompoundObjectScanModeEnum.All &&
        data.ScanSfxArchivedMode === CompoundObjectScanModeEnum.All &&
        data.ScanMailBasesMode === CompoundObjectScanModeEnum.All &&
        data.ScanPlainMailMode === CompoundObjectScanModeEnum.All &&
        data.ScanPackedMode === CompoundObjectScanModeEnum.All &&
        data.ScanOleMode === CompoundObjectScanModeEnum.All &&
        data.ScanHSMOptions.ActionType === HSMScanActionEnum.ScanEntireFile &&
        data.DeleteArchives === false ) {
        return 'protectionLevel.maxProtect';
    }
    else if (
        data.ScanObjects === true &&
        data.ScanType === ScanTypeEnum.AllObjects &&
        data.ScanSectors === true &&
        data.ScanStreams === true &&
        data.ScanArchived === true &&
        data.ScanSfxArchived === true &&
        data.ScanMailBases === false &&
        data.ScanPlainMail === false &&
        data.ScanPacked === true &&
        data.ScanOle === true &&
        data.UseTimeLimit === false &&
        data.UseSizeLimit === false &&
        data.InfectedFirstAction.ActionType === OdsActionEnum.Recommended &&
        data.InfectedSecondAction.ActionType === OdsActionEnum.Recommended &&
        data.SuspiciousFirstAction.ActionType === OdsActionEnum.Recommended &&
        data.SuspiciousSecondAction.ActionType === OdsActionEnum.Recommended &&
        data.UseAdvancedActions === false &&
        data.UseExcludeMasks === false &&
        data.UseExcludeThreats === false &&
        data.ScanOnlyNewAndChangedFiles === false &&
        data.ScanArchivedMode === CompoundObjectScanModeEnum.All &&
        data.ScanSfxArchivedMode === CompoundObjectScanModeEnum.All &&
        data.ScanPackedMode === CompoundObjectScanModeEnum.All &&
        data.ScanOleMode === CompoundObjectScanModeEnum.All &&
        data.ScanHSMOptions.ActionType === HSMScanActionEnum.ScanResidentPortion &&
        data.DeleteArchives === false ) {
        return 'protectionLevel.recomended';
    }
    else if (
        data.ScanObjects === true &&
        data.ScanType === ScanTypeEnum.ByFormat &&
        data.ScanSectors === true &&
        data.ScanStreams === true &&
        data.ScanArchived === false &&
        data.ScanSfxArchived === true &&
        data.ScanMailBases === false &&
        data.ScanPlainMail === false &&
        data.ScanPacked === true &&
        data.ScanOle === true &&
        data.UseTimeLimit === true &&
        data.TimeLimit === 60 &&
        data.UseSizeLimit === true &&
        data.SizeLimit === 8388608 &&
        data.InfectedFirstAction.ActionType === OdsActionEnum.Cure &&
        data.InfectedSecondAction.ActionType === OdsActionEnum.Remove &&
        data.SuspiciousFirstAction.ActionType === OdsActionEnum.Quarantine &&
        data.SuspiciousSecondAction.ActionType === OdsActionEnum.Report &&
        data.UseAdvancedActions === false &&
        data.UseExcludeMasks === false &&
        data.UseExcludeThreats === false &&
        data.ScanOnlyNewAndChangedFiles === true &&
        data.ScanHSMOptions.ActionType === HSMScanActionEnum.Skip &&
        data.DeleteArchives === false ) {
        return 'protectionLevel.maxSpeed';
    }

    return 'protectionLevel.custom';
}

module.exports = {
    CheckOdsLevel,
}